/* 

==========================================
   Folstag's Raiders Scenario, Player 2
==========================================

AI Script Writer:		Matthew Kingshott
Scenario Designer:		Matthew Kingshott
AI Script Editor:		Microsoft Development Environment 2003, Version 7.1.3088
Final Compile:			3:53 PM, 15th January 2004

Script Introduction
-------------------
						
This script manages the raiding cavalry sent by Folstag during the first segment
of the scenario, the creation of the cavalry is done via triggers and the plan
simply sends them into battle.

The second part of the script manages Folstag's base and his subsequent attacks
upon the completion of the first objective. The script uses existing units to
maintain and defend the camp while training plans exist to create enemy forces
to send against the player.

The Loki ravine which sends the wrath of Loki against the player is activated by
triggers and can be disabled by returning the 4 relics to the Loki temple.

Scenario Plot
-------------

This particular scenario joins the dwarven heroes Brokk & Eitri early in their
tales when they were but small villagers defending their home from the raids of
the Frost Giant Folstag.

*/

mutable int createSimpleDefendPlan(string name="default defend plan", int puid=-1, int number=-1, vector gatherPoint=vector(-1,-1,-1)) { }
mutable int createSimpleAttackPlanSingleGroup(string name="default attack plan", int playerID=-1, int puid1=-1, int number1=-1, vector gatherPoint=vector(-1,-1,-1)) { }
mutable int createSimpleAttackPlanTripleGroup(string name="default attack plan", int playerID=-1, int puid1=-1, int number1=-1, int puid2=-1, int number2=-1, int puid3=-1, int number3=-1, vector gatherPoint=vector(-1,-1,-1)) { }

void main(void)
{

// Identify that the script is active
aiEcho("Folstag's Raiders, Player 2, script running.");

// Display the map size e.g. 200m x 200m
aiEcho("Map size is ("+kbGetMapXSize()+" metres by "+kbGetMapZSize()+" metres)"); 

// Check where everybody is (just for analysis really, the script doesn't use the info it gets from doing this)
kbLookAtAllUnitsOnMap(); 

// Calculate map areas and grid squares
kbAreaCalculate(1200.0); 

// Allocate all resources to the root escrow account, this allows the plans to use any resources it can find
kbEscrowAllocateCurrentResources(); 

// Update debugger
aiEcho("All resources allocated to the root account!");

// Begin defence plans for Folstag's defenders.
createSimpleDefendPlan("Folstag's Camp Defence", cUnitTypeHeroNorse, 16, kbGetBlockPosition("3485"));

// Signal that we have reached the end of the AI Script
aiEcho("Thats all folks!");

}

void sendRaiders(int value=-1)
{

// Attack the player
createSimpleAttackPlanSingleGroup("Raiding Cavalry Attack", 1, cUnitTypeRaidingCavalry, 10, kbGetBlockPosition("3484"));

// Update the debugger
aiEcho("We are now launching the attack plan!");

}

void sendAttackers(int value=-1)
{

// Attack the player
createSimpleAttackPlanTripleGroup("Folstag Main Attack", 1, cUnitTypeFrostGiant, 3, cUnitTypeJarl, 5, cUnitTypeHuskarl, 5, kbGetBlockPosition("3484"));

// Update the debugger
aiEcho("We are now launching the attack plan!");

}

int createSimpleDefendPlan(string name="default training plan", int puid=-1, int number=-1, vector protectPoint=vector(-1,-1,-1))
{
   // Create a defend plan and assign the details of the plan to the variable "planID"
   int planID=aiPlanCreate(name, cPlanDefend);
  
   // If for some reason the above fails the code below discontinues the operation.
   if (planID < 0)
      return(false);
 
   // Add types of units that are in the plan
   aiPlanAddUnitType(planID, puid, 1, number, number);   
   
   // Set the priority of the plan
   aiPlanSetDesiredPriority(planID, 50);
   
   // Set the defence point
   aiPlanSetVariableVector(planID, cDefendPlanDefendPoint, 0, protectPoint);

   // Set the distance before they attempt to defend
   aiPlanSetVariableFloat(planID, cDefendPlanEngageRange, 0, 15);

   // Set the gather distance
   aiPlanSetVariableFloat(planID, cDefendPlanGatherDistance, 0, 15.0);

   // Set the initial defence position
   aiPlanSetInitialPosition(planID, protectPoint);

   // Set the stance to a defensive posture
   aiPlanSetUnitStance(planID, cUnitStanceDefensive);

   // Refresh the plan at this interval
   aiPlanSetVariableInt(planID, cDefendPlanRefreshFrequency, 0, 5);

   // The number of targets the plan will defend against (in order of priority)
   aiPlanSetNumberVariableValues(planID, cDefendPlanAttackTypeID, 2, true);

   // The target types to defend against (we could put walls in here)
   aiPlanSetVariableInt(planID, cDefendPlanAttackTypeID, 0, cUnitTypeUnit);
   aiPlanSetVariableInt(planID, cDefendPlanAttackTypeID, 1, cUnitTypeBuilding);
   
   // Activate the plan
   aiPlanSetActive(planID); 

   // Done.
   return(planID);
}

mutable int createSimpleAttackPlanSingleGroup(string name="default attack plan", int playerID=-1, int puid1=-1, int number1=-1, vector gatherPoint=vector(-1,-1,-1))
{
   // Create an attack plan and assign the details of the plan to the variable "planID"
   int planID=aiPlanCreate(name, cPlanAttack);
  
   // If for some reason the above fails the code below discontinues the operation.
   if (planID < 0)
      return(false);

   // The player to attack (usually player 1)
   aiPlanSetVariableInt(planID, cAttackPlanPlayerID, 0, playerID);

   // The number of target types that the plan will attack
   aiPlanSetNumberVariableValues(planID, cAttackPlanTargetTypeID, 3, true);
  
   // The types of targets the plan will attack (in order of priority)
   aiPlanSetVariableInt(planID, cAttackPlanTargetTypeID, 0, cUnitTypeUnit);
   aiPlanSetVariableInt(planID, cAttackPlanTargetTypeID, 1, cUnitTypeBuilding);
   aiPlanSetVariableInt(planID, cAttackPlanTargetTypeID, 2, cUnitTypeAbstractWall);

   // Set the gather point and gather point distance.
   aiPlanSetVariableVector(planID, cAttackPlanGatherPoint, 0, gatherPoint);
   aiPlanSetVariableFloat(planID, cAttackPlanGatherDistance, 0, 30.0);
  
   // Add the unit types to the plan.
   aiPlanAddUnitType(planID, puid1, 1, number1, number1);
   
   // Set the initial attack position.
   aiPlanSetInitialPosition(planID, gatherPoint);

   // Plan requires all needed units to work.
   aiPlanSetRequiresAllNeedUnits(planID, true);

   // Activate the plan.
   aiPlanSetActive(planID);
   
   // Done.
   return(planID);
}

mutable int createSimpleAttackPlanTripleGroup(string name="default attack plan", int playerID=-1, int puid1=-1, int number1=-1, int puid2=-1, int number2=-1, int puid3=-1, int number3=-1, vector gatherPoint=vector(-1,-1,-1))
{
   // Create an attack plan and assign the details of the plan to the variable "planID"
   int planID=aiPlanCreate(name, cPlanAttack);
  
   // If for some reason the above fails the code below discontinues the operation.
   if (planID < 0)
      return(false);

   // The player to attack (usually player 1)
   aiPlanSetVariableInt(planID, cAttackPlanPlayerID, 0, playerID);

   // The number of target types that the plan will attack
   aiPlanSetNumberVariableValues(planID, cAttackPlanTargetTypeID, 3, true);
  
   // The types of targets the plan will attack (in order of priority)
   aiPlanSetVariableInt(planID, cAttackPlanTargetTypeID, 0, cUnitTypeUnit);
   aiPlanSetVariableInt(planID, cAttackPlanTargetTypeID, 1, cUnitTypeBuilding);
   aiPlanSetVariableInt(planID, cAttackPlanTargetTypeID, 2, cUnitTypeAbstractWall);

   // Set the gather point and gather point distance.
   aiPlanSetVariableVector(planID, cAttackPlanGatherPoint, 0, gatherPoint);
   aiPlanSetVariableFloat(planID, cAttackPlanGatherDistance, 0, 30.0);
  
   // Add the unit types to the plan.
   aiPlanAddUnitType(planID, puid1, 1, number1, number1);
   aiPlanAddUnitType(planID, puid2, 1, number2, number2);
   aiPlanAddUnitType(planID, puid3, 1, number3, number3);
   
   // Set the initial attack position.
   aiPlanSetInitialPosition(planID, gatherPoint);

   // Plan requires all needed units to work.
   aiPlanSetRequiresAllNeedUnits(planID, true);

   // Activate the plan.
   aiPlanSetActive(planID);
   
   // Done.
   return(planID);
}